@@ -75,7 +75,7 @@ Status XGetSizeHints (
 
         if ((actual_type != XA_WM_SIZE_HINTS) ||
 	    (nitems < OldNumPropSizeElements) || (actual_format != 32)) {
-		if (prop != NULL) Xfree (prop);
+		Xfree (prop);
                 return(0);
 		}
 	hints->flags	  = (prop->flags & (USPosition|USSize|PAllHints));
@@ -124,7 +124,7 @@ XWMHints *XGetWMHints (
 
         if ((actual_type != XA_WM_HINTS) ||
 	    (nitems < (NumPropWMHintsElements - 1)) || (actual_format != 32)) {
-		if (prop != NULL) Xfree (prop);
+		Xfree (prop);
                 return(NULL);
 		}
 	/* static copies not allowed in library, due to reentrancy constraint*/
@@ -196,7 +196,7 @@ Status XGetIconSizes (
 	    (nitems < NumPropIconSizeElements) ||
 	    (nitems % NumPropIconSizeElements != 0) ||
 	    (actual_format != 32)) {
-		if (prop != NULL) Xfree (prop);
+		Xfree (prop);
                 return(0);
 		}
 
@@ -204,7 +204,7 @@ Status XGetIconSizes (
 
 	nitems /= NumPropIconSizeElements;
 	if (! (hp = hints = Xcalloc (nitems, sizeof(XIconSize)))) {
-	    if (prop) Xfree (prop);
+	    Xfree (prop);
 	    return 0;
 	}
 
@@ -239,7 +239,7 @@ Status XGetCommand (
     if (!XGetTextProperty (dpy, w, &tp, XA_WM_COMMAND)) return 0;
 
     if (tp.encoding != XA_STRING || tp.format != 8) {
-	if (tp.value) Xfree (tp.value);
+	Xfree (tp.value);
 	return 0;
     }
 
@@ -254,11 +254,11 @@ Status XGetCommand (
      * create a string list and return if successful
      */
     if (!XTextPropertyToStringList (&tp, &argv, &argc)) {
-	if (tp.value) Xfree (tp.value);
+	Xfree (tp.value);
 	return (0);
     }
 
-    if (tp.value) Xfree (tp.value);
+    Xfree (tp.value);
     *argvp = argv;
     *argcp = argc;
     return 1;
@@ -291,7 +291,7 @@ XGetTransientForHint(
 	return (1);
 	}
     *propWindow = None;
-    if (data) Xfree( (char *) data);
+    Xfree( (char *) data);
     return(0);
 }
 
@@ -315,14 +315,14 @@ XGetClassHint(
            return (0);
 
    if ( (actual_type == XA_STRING) && (actual_format == 8) ) {
-	len_name = strlen((char *) data);
+	len_name = (int) strlen((char *) data);
 	if (! (classhint->res_name = Xmalloc(len_name + 1))) {
 	    Xfree(data);
 	    return (0);
 	}
 	strcpy(classhint->res_name, (char *) data);
 	if (len_name == nitems) len_name--;
-	len_class = strlen((char *) (data+len_name+1));
+	len_class = (int) strlen((char *) (data+len_name+1));
 	if (! (classhint->res_class = Xmalloc(len_class + 1))) {
 	    Xfree(classhint->res_name);
 	    classhint->res_name = (char *) NULL;
@@ -333,6 +333,6 @@ XGetClassHint(
 	Xfree( (char *) data);
 	return(1);
 	}
-    if (data) Xfree( (char *) data);
+    Xfree( (char *) data);
     return(0);
 }
